#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['marsbots', 'marsbots.programs', 'marsbots.resources']

package_data = \
{'': ['*']}

install_requires = \
['cohere', 'requests', 'openai', 'python-dotenv', 'py-cord==2.0.0b1']

extras_require = \
{'dev': ['black', 'pytest', 'mypy', 'pre-commit', 'flake8']}

setup(name='marsbots',
      version='0.1.0',
      description='The core framework for cool Martian bots.',
      author=None,
      author_email='Jonathan Miller <jon@jmill.dev>',
      url=None,
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
     )
